# StatusFirst.pm (c) 2007 by Peter Watkins (peterw@tux.org) 
#
# This code is derived from code with the following copyright message
# and released under the same license terms:
#
# SliMP3 Server Copyright (C) 2001 Sean Adams, Slim Devices Inc.
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License,
# version 2.

use strict;

package Plugins::StatusFirst::Plugin;

use Slim::Control::Command;
use Slim::Control::Request;
use Slim::Hardware::IR;
use Slim::Utils::Strings qw (string);
use Slim::Utils::Misc qw( msg );

# after this much time, consider a button press a status query
my $statusDelayTime = 25 * $Slim::Hardware::IR::IRMINTIME;

# default behavior for players that have not used Shuffle/Repeat yet
# 1 == enabled, 0 == normal SlimServer behavior
my $defaultStatusFirstSetting = 1;

use vars qw($VERSION);
$VERSION = &rcsVersion();

# our strings
sub strings() { 
  	return '
PLUGIN_STATUSFIRST
	EN	StatusFirst
	
PLUGIN_STATUSFIRST_PLAY_TO_ENABLE
	EN	Press PLAY to enable StatusFirst

PLUGIN_STATUSFIRST_ADD_TO_REVERT
	EN	Press ADD for normal Repeat/Shuffle behavior
'};


# whether we need to add our callback function to the stack
my $callbackSet = 0;

# whether we're enabled (the callback remains in the function 
# stack after "disabling" the plugin, so we need to keep track of this)
my $pluginEnabled = 0;

# need to keep track of original button function
my $originalButtonCommand;

# keep track of last time we showed status
my %lastStatus;

# what the names are for Slim's repeat & shuffle strings, mapped to pref int values
my %stringNames = (
	'repeat' => ['REPEAT_OFF', 'REPEAT_ONE', 'REPEAT_ALL'],
	'shuffle' => ['SHUFFLE_OFF', 'SHUFFLE_ON_SONGS', 'SHUFFLE_ON_ALBUMS'],
);

# initialize
sub initPlugin {
	if ( $callbackSet == 0 ) {
		# use the cleaner addDispatch technique
		$::d_plugins && &SF_msg("wrapping button functions\n");
		$originalButtonCommand = Slim::Control::Request::addDispatch(['button','_buttoncode','_time','_orFunction'],[1, 0, 0, \&StatusFirst_buttonCommand]);
		if ( (!defined($originalButtonCommand)) || (ref($originalButtonCommand ) ne 'CODE') ) {
			$::d_plugins && &SF_msg("problem wrapping button command!\n");
		}
		$callbackSet = 1; 
	}
	# note that we should act
	$pluginEnabled = 1;
}

sub shutdownPlugin {
	# we should not act
	$pluginEnabled = 0;
}

sub enabled {
	if ( substr($::VERSION,0,3) gt '6.5' ) {
		&SF_msg("This plugin has not been tested with SlimServer newer than 6.5.x\n");
	}
	# this uses the Slim::Control::Request API that's in 6.5.x
	return ($::VERSION ge '6.5');
}

sub rcsVersion() {
	my $RcsVersion = '$Revision: 1.5 $';
	$RcsVersion =~ s/.*:\s*([0-9\.]*).*$/$1/;
	return $RcsVersion;
}

sub getDisplayName() {
	return 'PLUGIN_STATUSFIRST';
}

sub setMode() {
	my $client = shift;
	$client->lines(\&lines);
}

# get the client's setting, and intialize if needed
sub clientStatusFirstSetting($) {
	my $client = shift;
	my $val = Slim::Utils::Prefs::clientGet($client,'plugin-StatusFirst-enabled');
	if (! defined($val) ) {
		$val = $defaultStatusFirstSetting;
		Slim::Utils::Prefs::clientSet($client,'plugin-StatusFirst-enabled',$val);
	}
	return $val;
}

# whether we shouldshow status instead of changing
sub shouldShowStatus($$) {
	my ($id,$buttoncode) = @_;
	if ( ! defined($lastStatus{"${id}-${buttoncode}"}) ) {
		return 1;
	}
	my $now  = Time::HiRes::time();
	my $delay = ($now - $lastStatus{"${id}-${buttoncode}"});
	if ( $delay > $statusDelayTime ) {
		return 1;
	}
	return 0;
}

# update the timer for this client for this button
sub updateStatus($$) {
	my ($id,$buttoncode) = @_;
	my $now  = Time::HiRes::time();
	$lastStatus{"${id}-${buttoncode}"} = $now;
}

# our wrapper function
sub StatusFirst_buttonCommand {
	my @args = @_;
	my $request = $args[0];
	my $buttoncode = $request->getParam('_buttoncode');
	# we only consider something different for Repeat and Shuffle
	if ( defined($buttoncode) && 
	     (($buttoncode eq 'shuffle_toggle') || 
	      ($buttoncode eq 'repeat_toggle') || 
	      ($buttoncode eq 'sleep') || 
	      ($buttoncode =~ m/^volume_(up|down)$/)) && 
	     ($pluginEnabled == 1) ) {
		my $client = $request->client();
		my $id = $client->id();
		if ( (&clientStatusFirstSetting($client) == 1) && (&shouldShowStatus($id,$buttoncode) == 1) ) {
			$::d_plugins && &SF_msg("StatusFirst enabled for $id, showing $buttoncode status\n");
			&showStatus($client,$buttoncode);
			&updateStatus($id,$buttoncode);
			$request->setStatusDone;
			return;
		}
		# else call the original function
		&updateStatus($id,$buttoncode);
	}
	$::d_plugins && &SF_msg("calling original button $buttoncode function\n");
	return &$originalButtonCommand(@args);
}

# show the current status of Shuffle or Repeat
sub showStatus($$) {
	my ($client,$buttoncode) = @_;
	my $line1 = string('PLUGIN_STATUSFIRST');
	my $statusMessage;
	if ( $buttoncode =~ m|^volume_| ) {
		$client->mixerDisplay('volume');
		return;
	}
	if ( $buttoncode eq 'sleep' ) {
		$statusMessage = $client->prettySleepTime;
		if ( $statusMessage =~ m|^\s*$| ) {
			# no timer set
			$statusMessage = string('CANCEL_SLEEP');
		}
	} else {
		my $prefname = $buttoncode;
		$prefname =~ s/_toggle$//;
		# get pref
		my $pref = $client->prefGet($prefname);
		# get pref string
		my @statusStrings = @{$stringNames{$prefname}};
		$statusMessage = string($statusStrings[$pref]);
	}
	# show status
	$client->showBriefly($line1,"=${statusMessage}",1,0,1);
}

# player UI functions
my %functions = (
	'up' => sub  {
		my $client = shift;
		$client->bumpUp($client);
	},
	'down' => sub  {
	    my $client = shift;
		$client->bumpDown($client);
	},
	'left' => sub  {
		my $client = shift;
		Slim::Buttons::Common::popModeRight($client);
	},
	'right' => sub  {
		my $client = shift;
		$client->bumpRight($client);
	},
	'play' => sub {
		my $client = shift;
		Slim::Utils::Prefs::clientSet($client,'plugin-StatusFirst-enabled',1);
		# refresh the display with current change option
		$client->lines(\&lines);
		$client->update();
	},
	'add' => sub {
		my $client = shift;
		Slim::Utils::Prefs::clientSet($client,'plugin-StatusFirst-enabled',0);
		# refresh the display with current change option
		$client->lines(\&lines);
		$client->update();
	}
);

sub lines {
	my $client = shift;
	my ($line1, $line2);
	$line1 = string('PLUGIN_STATUSFIRST');
	if ( &clientStatusFirstSetting($client) == 1 ) {
		$line2 = string('PLUGIN_STATUSFIRST_ADD_TO_REVERT');
	} else {
		$line2 = string('PLUGIN_STATUSFIRST_PLAY_TO_ENABLE');
	}
	return ($line1, $line2);
}

sub getFunctions() {
	return \%functions;
}

sub SF_msg {
	my $message = shift;
	msg("StatusFirst: $message");
}

1;

